@@ -1,12 +1,12 @@
 #!perl -w
 package version;
 
-use 5.005_04;
+use 5.006002;
 use strict;
 
 use vars qw(@ISA $VERSION $CLASS $STRICT $LAX *declare *qv);
 
-$VERSION = 0.9906;
+$VERSION = 0.9907;
 $CLASS = 'version';
 
 # !!!!Delete this next block completely when adding to Perl core!!!!
